importClass(Packages.java.security.KeyStore);
importClass(Packages.java.security.Signature);

var CASD_VERSION = "-1-6-1";

var CAServiceDeskWS = BaseClass.extend({

    LOG_SOURCE: "caservicedeskws.js: ",

    CA_SOAP_VERSION: "1_1",

    SERVICE_DESK_WSDL_DEFN: "integrationservices/caservicedesk" + CASD_VERSION + "/caservicedesk" + CASD_VERSION + "/soap/CA-Service-Desk-soapui-project.xml",

    /**
     * Constructor
     * @param endPoint SOAP EndPoint for CA Service Desk Manager
     * @param sid session id for the webservices session
     */
    init: function(wsutil, endPoint)
    {
        this.log = new Logger(this.LOG_SOURCE);

        this.wsutil = wsutil;
        this.endPoint = endPoint;
        this.sid = null;

        this.log.info("CAServiceDeskWS constructed. endPoint [" + this.endPoint + "]");
    },

    /**
     * Creates a new activity log entry for the selected incident
     * @param objectHandle id of the incident to annotate
     * @param creator id of the user updating the incident log
     * @param description message to annotate
     * @param logType type of log entry to create
     * @param internal true if this log is internal only, false otherwise
     */
    annotate: function(objectHandle, creator, description, logType, internal)
    {
        var msg = new SOAPMessage(this.SERVICE_DESK_WSDL_DEFN, "createActivityLog", "createActivityLog", this.CA_SOAP_VERSION);

        // Set the parameter values
        msg.setParameter("sid", this.getSessionId());
        msg.setParameter("creator", creator);
        msg.setParameter("objectHandle", objectHandle);
        msg.setParameter("description", description);
        msg.setParameter("logType", !isEmpty(logType) ? logType : "LOG");
        msg.setParameter("logType", !isEmpty(logType) ? logType : "LOG");
        msg.setParameter("timeSpent", "0");
        msg.setParameter("internal", !isEmpty(internal) ? internal : "false");

        // Post the message to the SOAP endpoint
        var response = this.wsutil.sendReceive(this.endPoint, msg);
        return this.wsutil.unmarshall(response, new Object());
    },

    /**
     * Update the incident with the new status and assignee (if applicable)
     * @param incidentId id of the incident to update
     * @param status new status of the incident
     * @param additionalTokens Collection of attributes to be added to the SOAP Request
     * @param objectHandle ID of the object being updated
     */
    updateTicket: function(incidentId, status, additionalTokens, objectHandle)
    {
        var attrVals = "<string>status</string><string>" + status.toUpperCase() + "</string>";

        attrVals += getAttributeValues(additionalTokens);

        var msg = new SOAPMessage(this.SERVICE_DESK_WSDL_DEFN, "updateObject", "updateObject", this.CA_SOAP_VERSION);

        msg.setParameter("sid", this.getSessionId());
        msg.setParameter("objectHandle", objectHandle);
        msg.setParameter("attrVals", attrVals, false);
        msg.setParameter("attributes", "", false);

        // Post the message to the SOAP endpoint
        var response = this.wsutil.sendReceive(this.endPoint, msg);
        return this.wsutil.unmarshall(response, new Object());
    },

    /**
     * Create a new ticket based on the role of the user creating the ticket
     * @param customerHandle id of the user creating the ticket
     * @param description summary of the ticket being created
     */
    createQuickTicket: function(customerHandle, description)
    {
        var msg = new SOAPMessage(this.SERVICE_DESK_WSDL_DEFN, "createQuickTicket", "createQuickTicket", this.CA_SOAP_VERSION);

        msg.setParameter("sid", this.getSessionId());
        msg.setParameter("customerHandle", customerHandle);
        msg.setParameter("description", description);

        // Post the message to the SOAP endpoint
        var response = this.wsutil.sendReceive(this.endPoint, msg);
        return this.wsutil.unmarshall(response, new Object());
    },

    /**
     * Logs the user into CA Service Desk
     * @param userId ID of user to login for this session
     * @param password String password for the user
     * @return session id for further requests
     */
    login: function(userId, password)
    {
        this.log.info("Enter - login");

        
          var msg = new SOAPMessage(this.SERVICE_DESK_WSDL_DEFN, "login", "login", this.CA_SOAP_VERSION);

          // Set the parameter values
          msg.setParameter("username", userId);
          msg.setParameter("password", password);

          // Post the message to the SOAP endpoint
          var response = this.wsutil.sendReceive(this.endPoint, msg);
          this.sid = response.*::loginResponse.loginReturn;

          if (this.sid <= 0)
          {
              throw "Unable to create WebService Session";
          }

        this.log.info("sid [" + this.sid + "]");
        this.log.info("Exit - login");
    },

    /**
     * Logs the user into CA Service Desk using the CA Service Desk Manager Certificate
     * @param userId ID of user that will be impersonated if impersonation is enabled
     * @return session id for further requests
     */
    loginServiceManaged: function(userId)
    {
        // Get the appropriate Keystore instance to load the .p12 file
        var ks = KeyStore.getInstance("PKCS12");

        // Create a password to be used when extracting the private key. Same as the access policy.
        var privateKeyPassword = ACCESS_POLICY.split("");

        // Load the certificate
        var dir = ACCESS_POLICY_FILE_PATH.lastIndexOf("/") != -1 ? ACCESS_POLICY_FILE_PATH : ACCESS_POLICY_FILE_PATH + "/";
        ks.load(new FileInputStream(dir + ACCESS_POLICY + ".p12"), privateKeyPassword);

        // Extracting the private key, the first parameter is the alias associated
        // with the key which is "servicedesk <accessPolicy>"(all lower case);
        // the second parameter is the password to extract the key (this defaults to
        // the Access Policy name as well ... when using pdm_pki utility)
        var key = ks.getKey("servicedesk " + ACCESS_POLICY.toLowerCase(), privateKeyPassword);

        // Creating an instance of the signature class to create a digital signature
        // of the private key...must use the SHA1withRSA algorithm
        var s = Signature.getInstance("SHA1withRSA");

        // Initializing the signature with the private key
        s.initSign(key);

        // Updating the signature with the access policy
        var b = new java.lang.String(ACCESS_POLICY).getBytes();
        s.update(b, 0, b.length);

        // Creating the digital signature
        var sig = s.sign();

        // Converting the signature to BASE64 text format to pass into loginServiceManaged
        var encryption = Base64.encode(sig);

        var msg = new SOAPMessage(this.SERVICE_DESK_WSDL_DEFN, "loginServiceManaged", "loginServiceManaged", this.CA_SOAP_VERSION);

        // Set the parameter values
        msg.setParameter("policy", ACCESS_POLICY);
        msg.setParameter("encrypted_policy", encryption);

        // Post the message to the SOAP endpoint
        var response = this.wsutil.sendReceive(this.endPoint, msg);
        this.sid = response.*::loginServiceManagedResponse.loginServiceManagedReturn;

        if (this.sid <= 0)
        {
            throw "Unable to create WebService Session";
        }

        // If impersonation is enabled, impersonate the selected user.
        if (ENABLE_IMPERSONATE)
        {
            this.impersonate(userId);
        }
    },

    /**
     * Logs the user into CA Service Desk impersonating the requested user
     * @param userId name of user to impersonate
     * @return session id for further requests
     */
    impersonate: function(userId)
    {
        var msg = new SOAPMessage(this.SERVICE_DESK_WSDL_DEFN, "impersonate", "impersonate", this.CA_SOAP_VERSION);

        // Set the parameter values
        msg.setParameter("sid", this.sid);
        msg.setParameter("userid", userId);

        // Post the message to the SOAP endpoint
        var response = this.wsutil.sendReceive(this.endPoint, msg);
        this.sid = response.*::impersonateResponse.impersonateReturn;

        if (this.sid <= 0)
        {
            throw "Unable to create WebService Session";
        }
    },

    /**
     * Logs the current user session out of CA Service Desk
     */
    logout: function()
    {
        if (this.sid != null)
        {
            var msg = new SOAPMessage(this.SERVICE_DESK_WSDL_DEFN, "logout", "logout", this.CA_SOAP_VERSION);

            // Set the parameter values
            msg.setParameter("sid", this.sid);

	    // Post the message to the SOAP endpoint
	    this.wsutil.sendReceive(this.endPoint, msg);

            this.sid = null;
        }
    },

    /**
     * Session ID for the current WS Session.
     * If the session is not we retrive it by logging in.
     */
    getSessionId: function()
    {
        if (this.sid == null)
        {
            this.login(SERVICE_DESK_USER, SERVICE_DESK_PASSWORD);
        }

        return this.sid;
    },

    /**
     * Retrieves the requested information from Service Desk
     * @param objectType type of Object to retrieve
     * @param whereClause filter that limits the results retrieved
     * @param maxRows specifies the maximum number of records that will be returned
     * @param attributes array of attributes that will be returned in the resultset
     * @return XML fragment containing the requested data
     */
    doSelect: function(objectType, whereClause, maxRows, attributes)
    {
        var msg = new SOAPMessage(this.SERVICE_DESK_WSDL_DEFN, "doSelect", "doSelect", this.CA_SOAP_VERSION);

        // Set the parameter values
        msg.setParameter("sid", this.getSessionId());
        msg.setParameter("objectType", objectType);
        msg.setParameter("whereClause", whereClause);
        msg.setParameter("maxRows", maxRows);
        msg.setParameter("attributes", createListElements("string", attributes), false);

        // Post the message to the SOAP endpoint
        var response = this.wsutil.sendReceive(this.endPoint, msg);

        response = new XML(this.wsutil.formatStringForE4X(response.*::doSelectResponse.doSelectReturn));
	var udsObjectList = [];
	for each (item in response.UDSObject)
	{
	    var udsObject = this.unmarshallUdsObject(item, new Object());
	    udsObjectList.push(udsObject);
	}

	return udsObjectList;
    },

    /**
     * Basically, since we know what a uds object looks like get rid of all the abstractions (ie Attribute ...).
     * Assumption:
     *   1. No attributes will have the same name.
     */
    unmarshallUdsObject: function(xml, object)
    {
//        this.log.debug("Enter - unmarshallUdsObject");

		object['Handle'] = xml.Handle;
		for each (var attribute in xml.Attributes.*)
		{
//			this.log.debug("Creating property: " + attribute.AttrName + " = " + attribute.AttrValue);
			object[attribute.AttrName] = attribute.AttrValue;
		}

//        this.log.debug("Exit - unmarshallUdsObject");
		return object;
    },
    
    /**
     * Retrieves a list of objectType from Service Desk
     * @param objectType type of Object to retrieve
     * @param whereClause similar to SQL WHERE statement to limit objectTypes included in list
     * @return XML fragment that consists of the doQueryReturn element and its children,
     *         which is a list handle and length e.g
     *
     * <doQueryResponse xmlns="http://www.ca.com/UnicenterServicePlus/ServiceDesk">
     *   <doQueryReturn xmlns="">
     *     <listHandle>1508114219</listHandle>
     *     <listLength>13</listLength>
     *   </doQueryReturn>
     * </doQueryResponse>
     *
     */
    doQuery: function(objectType, whereClause)
    {
        this.log.info("Enter - doQuery");

        var msg = new SOAPMessage(this.SERVICE_DESK_WSDL_DEFN, "doQuery", "doQuery", this.CA_SOAP_VERSION);

        // Set the parameter values
        msg.setParameter("sid", this.getSessionId());
        msg.setParameter("objectType", objectType);
        msg.setParameter("whereClause", whereClause);

        // Post the message to the SOAP endpoint
        var response = this.wsutil.sendReceive(this.endPoint, msg);
        response = this.wsutil.unmarshall(response.*::doQueryResponse.doQueryReturn, new Object());

        this.log.info("Exit - doQuery");
        return response;
    },

    /**
     * Retrieves a list of objectType from Service Desk
     * @param handles array of list handle reference ids
     * @return XML fragment that consists of the doQueryReturn element and its children
     */
    freeListHandles: function(handles)
    {
        var msg = new SOAPMessage(this.SERVICE_DESK_WSDL_DEFN, "freeListHandles", "freeListHandles", this.CA_SOAP_VERSION);

        // Set the parameter values
        msg.setParameter("sid", this.getSessionId());
        msg.setParameter("handles", createListElements("integer", handles), false);

        // Post the message to the SOAP endpoint
        this.wsutil.sendReceive(this.endPoint, msg);
    },

    /**
     * Gets the values of the list created by a doQuery call
     * @param listHandle listHandle returned from doQuery
     * @param startIndex index within the list to start
     * @param endIndex index within the list to end
     * @param attributes attributes of the objectType to return
     * @return details
     */
    getListValues: function(listHandle, startIndex, endIndex, attributeNames)
    {
        this.log.info("Enter - getListValues");
        this.log.info("getListValues - listHandle [" + listHandle + "], startIndex [" + startIndex + "], endIndex [" + endIndex + "], attributeNames [" + attributeNames + "]");

        var msg = new SOAPMessage(this.SERVICE_DESK_WSDL_DEFN, "getListValues", "getListValues", this.CA_SOAP_VERSION);

        // Set the parameter values
        msg.setParameter("sid", this.getSessionId());
        msg.setParameter("listHandle", listHandle);
        msg.setParameter("startIndex", startIndex);
        msg.setParameter("endIndex", endIndex);
        msg.setParameter("attributeNames", createListElements("string", attributeNames), false);

        // Post the message to the SOAP endpoint
        var response = this.wsutil.sendReceive(this.endPoint, msg);

		// Strip the xml prolog from the xml document returned so we can use e4x to work with it
        response = new XML(this.wsutil.formatStringForE4X(response.*::getListValuesResponse.getListValuesReturn));

		// For each item in the list use e4x to convert it from an xml fragment to a javascript object.
		var udsObjectList = [];
		for each (item in response.UDSObject)
		{
			var udsObject = this.unmarshallUdsObject(item, new Object());
			udsObjectList.push(udsObject);
		}

		this.log.info("Exit - getListValues");
		return udsObjectList;
    },

    /**
     * Retrieves the GUID for the specified user
     * @param userId name of the user to lookup
     * @return user GUID
     */
    getHandleForUserId: function(userId)
    {
        if (userId == null)
        {
	    return null;
        }

	var msg = new SOAPMessage(this.SERVICE_DESK_WSDL_DEFN, "getHandleForUserid", "getHandleForUserid", this.CA_SOAP_VERSION);

	// Set the parameter values
	msg.setParameter("sid", this.getSessionId());
	msg.setParameter("userID", userId);

	// Post the message to the SOAP endpoint
	var response = this.wsutil.sendReceive(this.endPoint, msg);
	return this.wsutil.unmarshall(response.*::getHandleForUseridResponse.getHandleForUseridReturn, new Object());
    },

    /**
     * Retrieves the contact details for the specified contact ID
     * @param contactId id of the contact to lookup
     * @return contact information
     */
    getContact: function(contactId)
    {
        if (contactId == null)
        {
	    return null;
        }

	var msg = new SOAPMessage(this.SERVICE_DESK_WSDL_DEFN, "getContact", "getContact", this.CA_SOAP_VERSION);

	// Set the parameter values
	msg.setParameter("sid", this.getSessionId());
	msg.setParameter("contactId", contactId);

	// Post the message to the SOAP endpoint
	var response = this.wsutil.sendReceive(this.endPoint, msg);
        return this.wsutil.unmarshall(response.*::getContactResponse.getContactReturn, new Object());
    },

    /**
     * Retrieves the related list for the object handle
     * @param objectHandle id of the object the list relates to
     * @param listName name of the list that is being retrieved
     * @return list information to allow retrieval of the values
     */
    getRelatedList: function(objectHandle, listName)
    {
        var msg = new SOAPMessage(this.SERVICE_DESK_WSDL_DEFN, "getRelatedList", "getRelatedList", this.CA_SOAP_VERSION);

        // Set the parameter values
        msg.setParameter("sid", this.getSessionId());
        msg.setParameter("objectHandle", objectHandle);
        msg.setParameter("listName", listName);

        // Post the message to the SOAP endpoint
        var response = this.wsutil.sendReceive(this.endPoint, msg);
        return this.wsutil.unmarshall(response, new Object());
    },

    /**
     * Retrieves the transitions that have been defined for the workflow
     * @param objectHandle identifies the handle for a ticket or status
     * @param ticketFactory Identifies the factory of the ticket. Valid values are only "cr" for Request,
     *        "in" for Incident, "pr" for Problem, "chg" for Change Order and "iss" for Issue.
     * @return all of the possible transition values for the status of a ticket
     */
    getValidTransitions: function(objectHandle, ticketFactory)
    {
        var msg = new SOAPMessage(this.SERVICE_DESK_WSDL_DEFN, "getValidTransitions", "getValidTransitions", this.CA_SOAP_VERSION);

        // Set the parameter values
        msg.setParameter("sid", this.getSessionId());
        msg.setParameter("handle", objectHandle);
        msg.setParameter("ticketFactory", ticketFactory);

        // Post the message to the SOAP endpoint
        var response = this.wsutil.sendReceive(this.endPoint, msg);
        response = new XML(this.wsutil.formatStringForE4X(response.*::getValidTransitionsResponse.getValidTransitionsReturn));
        return this.wsutil.unmarshall(response, new Object());
    }

});
