/**
 * Helper class that will parse a phone number string and set the properties in the Device class to be synchronized
 * with xMatters.
 *
 */
var PHONE_NUMBER = BaseClass.extend({

    LOG_SOURCE: "phonenumber.js: ",

    /**
     * -----------------------------------------------------------------------------------------------------------------
     * Constructor
     *
     * @param endPoint - SOAP EndPoint URL
     * -----------------------------------------------------------------------------------------------------------------
     */
    init: function()
    {
        this.log = new Logger(this.LOG_SOURCE);
        this.log.debug("PHONE_NUMBER constructed.");
    },
    /**
     * -----------------------------------------------------------------------------------------------------------------
     * setVoicePhoneOrPager
     *
     * Parse the phone number string into its component tokens and update the xMatters device provided. The device must
     * be a Voice Device or Numeric Pager Device, and has the following fields:
     *
     * - countryCodeOverride (optional)
     * - areaCode (optional, but area codes from CA SDM must go in this field)
     * - number (mandatory)
     * - extension (optional)
     *
     * NOTE: countryCodeOverride is a string value that uses the "ISO 3166-1 alpha-2" 2-letter country codes.
     *        For example, see http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2
     * -----------------------------------------------------------------------------------------------------------------
     */
    setVoicePhoneOrPager: function(user, voiceDevice, sdmPhoneNumber)
    {
        this.log.debug("Enter - setVoicePhoneOrPager");
        this.log.debug("sdmPhoneNumber [" + sdmPhoneNumber + "]");


        // local numbers e.g. 250 4826, 250-4826
        var rexNumber = /^(\d+)\D+(\d+)$/;

        // 10 digit numbers such as 250 123 4567, (250) 123 4567, 250 123 4567, 250.123.4567
        var rexAreaCodeNumber = /^\D*(\d+)\D+(\d+)\D+(\d+)$/;

        // As above, prefixed with a 1
        var rexPrefixAreaCodeNumber = /^1\D+(\d+)\D+(\d+)\D+(\d+)$/;


        voiceDevice.isDelete = isEmpty(sdmPhoneNumber);

        // If the SDM contact has a phone number, see if it matches any of the regex patterns above.

        if (!isEmpty(sdmPhoneNumber))
        {
            if ((tokens = rexNumber.exec(sdmPhoneNumber)) != null)
            {
                log.debug("[" + tokens[1] + "], [" + tokens[2] + "]");

                voiceDevice.number = tokens[1] + tokens[2];
            }
            else if ((tokens = rexPrefixAreaCodeNumber.exec(sdmPhoneNumber)) != null)
            {
                log.info("[" + tokens[1] + "], [" + tokens[2] + "], [" + tokens[3] + "]");

                // Get the ISO country code for the dialing prefix of 1 that is part of this pattern.
                // For more general number formats that include dialing prefixes for other countries,
                // the mapping defined in countryCodes (configuration.js) will need to be expanded.
                
                voiceDevice.countryCodeOverride = this.isoCountryCodeFor("1");

                voiceDevice.areaCode = tokens[1];
                voiceDevice.number = tokens[2] + tokens[3];
            }
            else if ((tokens = rexAreaCodeNumber.exec(sdmPhoneNumber)) != null)
            {
                log.info("[" + tokens[1] + "], [" + tokens[2] + "], [" + tokens[3] + "]");

                voiceDevice.areaCode = tokens[1];
                voiceDevice.number = tokens[2] + tokens[3];
            }
            else
            {
                var msg = user.targetName + "|" + voiceDevice.deviceName + ": cannot parse number [" + sdmPhoneNumber + "]";
                this.log.warn("setVoicePhoneOrPager - " + msg);
                warningList.add(msg);
            }
        }

        this.log.debug("makeVoiceDevice - result " + voiceDevice);
        this.log.debug("Exit - setVoicePhoneOrPager");

        return voiceDevice;
    },

    /**
     * -------------------------------------------------------------------------------------------------------------
     * isoCountryCodeFor
     *
     * @param prefix - the international dialing prefix for the country
     *
     * Returns the ISO 3166-2 2-character country code for the first country in the list in countrycodes.js
     * whose international dialing prefix is provided.
     *
     * Returns:
     *
     * - the first matching country if one or more matches are found
     * - an empty string if the prefix supplied is itself empty or no match is found
     * -------------------------------------------------------------------------------------------------------------
     */
    isoCountryCodeFor: function(prefix)
    {
        this.log.debug("Enter - isoCountryCodeFor");
        this.log.debug("isoCountryCodeFor - prefix [" + prefix + "]");

        var countryCode = "";

        if (!isEmpty(prefix))
        {
            // Get the ISO code for the first country in the XML list that has the specified dialing prefix.
            var countryMatches = countryCodes.country.(dialingPrefix == String(prefix));

            if (!isEmpty(countryMatches))
            {
                countryCode = countryMatches[0].ISO_3166_1;
            }
            else
            {
                this.log.info("No match for country with dialing prefix [" + prefix + "]")
            }
        }

        this.log.debug("countryCode [" + countryCode + "]");
        this.log.debug("Exit - isoCountryCodeFor");

        return String(countryCode);
    },

});

