var groupsProcessed;

/**
 * --------------------------------------------------------------------------------------------------------------------
 * processCASDGroups
 *
 * Transfer groups and their memberships from CA SDM to xMatters.
 *
 * @param caServiceDeskWS instance of the CA Service Desk Web Service object
 * --------------------------------------------------------------------------------------------------------------------
 */
function processCASDGroups(caServiceDeskWS, qualification, isDelete, isUpdateOnly)
{
    log.info("Enter - processCASDGroups");
    log.info("processCASDGroups. isDelete [" + isDelete + "], qualification [" + qualification + "], chunkSize [" + CHUNK_SIZE + "], isUpdateOnly [" + isUpdateOnly + "]");

    groupsProcessed = 0;

    // doQuery will return a list Handle and the length of the list
    var doQueryReturn = caServiceDeskWS.doQuery(OBJECT_TYPE_CONTACT, qualification);

    var callback = function(udsObject)
    {
        tryProcessOneCASDGroup(caServiceDeskWS, udsObject, null, isDelete, isUpdateOnly);
    };

    iterateQueryList(caServiceDeskWS, doQueryReturn, ATTRIBUTES_CNT_OBJECT, callback);

    log.info("processCASDGroups - processed " + groupsProcessed + " groups");
    log.info("Exit - processCASDGroups");
}

/**
 * ---------------------------------------------------------------------------------------------------------------------
 * tryProcessOneCASDGroup
 *
 * Add/update or delete a single Remedy support group in xMatters. If anything goes wrong at this level, record it in the
 * failure list, but continue processing
 * ---------------------------------------------------------------------------------------------------------------------
 */
function tryProcessOneCASDGroup(caServiceDeskWS, groupObject, userDataSyncHelper, isDelete, isUpdateOnly)
{
    log.info("Enter - tryProcessOneCASDGroup");
    var group = null;

    var groupName = groupObject.last_name;
    log.info("Processing group [" + groupName + "]");

    try
    {
        if (isDelete)
        {
            group = new Group(groupName);
        }
        else
        {
            // Get the list of users who are members of this group
            var supportGroupMemberList = getGroupMemberList(caServiceDeskWS, groupObject);

            group = makeGroupForAddUpdate(groupObject, supportGroupMemberList, userDataSyncHelper, isUpdateOnly);
        }

        // syncGroup will do a delete if the 2nd argument is true
        (new xMattersGroupSync(syncList)).syncGroup(group, isDelete);

        groupsProcessed++;
        log.info("Finished processing group [" + groupName + "]");
    }
    catch (e)
    {
        log.error("tryProcessOneCASDGroup - while processing group [" + groupName + "] got exception [" + e.toString() + "]");
        failedList.add((isDelete ? "Delete" : "Add/Update") + " Group: " + groupName + " failed to synchronize: " + e.toString());
    }

    log.info("Exit - tryProcessOneCASDGroup");
}

/**
 * ---------------------------------------------------------------------------------------------------------------------
 * makeGroupForAddUpdate
 *
 * Load a single Remedy support group and its associated team / membership information.
 * ---------------------------------------------------------------------------------------------------------------------
 */
function makeGroupForAddUpdate(groupObject, casdGroupMemberList, userDataSyncHelper, isUpdateOnly)
{
    log.info("Enter - makeGroupForAddUpdate");

    var groupName = groupObject.last_name;
    log.info("Create xMatters sync group object for [" + groupName + "], isUpdateOnly [" + isUpdateOnly + "]");
    log.debug("casdGroupMemberList: " + casdGroupMemberList);

    var group = new Group(groupName);

     group.updateOnly = isUpdateOnly;

    // Some group properties are not derived from CA SDM information. These are only given initial values when the group
    // is created. After that, the values in xMatters are preserved during updates.
    if (!group.groupExists)
    {
        group.externallyOwned = EXTERNALLY_OWN_GROUPS;

        // The default supervisor is defined in configuration.js and depends on whether xMatters has
        // merged administrator accounts or not.
        group.supervisors = (new SupervisorMapper()).getSupervisorsForGroup(group);
        log.debug("New group object [" + groupName + "] is externallyOwned: " + group.externallyOwned);
    }
    else 
    {
       log.debug("Group object [" + groupName + "] does exist in xMatters");
    }
    

    // The remainder of the user properties are based on CA SDM information at the time of the update

    var deleteFlag = groupObject.delete_flag;

    group.active = !isEmpty(deleteFlag) && deleteFlag == "0";
    //group.description = groupObject.Description;

    group.coverage.groupName = group.name;
    group.team.groupName = group.name;
    group.team.externallyOwned = group.externallyOwned;
    
    // set coverage and team names
    group.team.name = DEFAULT_TEAM_NAME;
    group.team.teamName = group.team.name;
    group.team.teamDescription = DEFAULT_TEAM_DESC;
    group.coverage.name = DEFAULT_COVERAGE_NAME;
    group.coverage.teamName = group.team.teamName;

    casdGroupMemberList.forEach(function(casdGroupMember)
    {
        var userLoginId = "" + casdGroupMember;
        syncGroupMember = new GroupMember();
        syncGroupMember.targetName = userLoginId;
        group.team.members.put(userLoginId, syncGroupMember);
    });

    log.debug("Group to sync is: " + group.toString());
    log.info("Exit - makeGroupForAddUpdate");
    return group;
}

/**
 * ---------------------------------------------------------------------------------------------------------------------
 * getGroupMemberList
 *
 * Query CA SDM for members of a group
 *
 * return A javascript array of user id who are a member of the group
 * ---------------------------------------------------------------------------------------------------------------------
 */
function getGroupMemberList(caServiceDeskWS, groupObject)
{
    log.info("Enter - getGroupMemberList");

    var membersList = [];

    var groupId = groupObject.id;

    log.debug("getGroupMemberList - groupId [" + groupId + "]");

    var qualification = makeGroupMembersWhereClause(String(groupId));

    log.info("getGroupMemberList - qualification [" + qualification + "]");

    // doQuery will return a list Handle and the length of the list
    var doQueryReturn = caServiceDeskWS.doQuery(OBJECT_TYPE_GROUP_MEMBER, qualification);
    var callback = function(udsObject)
    {
        membersList.push(udsObject["member.userid"]);
    };

    iterateQueryList(caServiceDeskWS, doQueryReturn, ATTRIBUTES_GRPMEM_OBJECT, callback);

    log.info("getGroupMemberList - membersList [" + membersList + "]");
    log.info("Exit - getGroupMemberList");
    return membersList;
}

/**
 * ---------------------------------------------------------------------------------------------------------------------
 * iterateQueryList
 *
 * Given the results of a doQuery (ie a list handle) call getListValues to obtain the entries in the list.
 * A call back should be provided to perform an operation on each entry returned.
 * ---------------------------------------------------------------------------------------------------------------------
 */
function iterateQueryList(caServiceDeskWS, doQueryReturn, attributes, callback)
{
    log.info("Enter - iterateQueryList");
    var listLength = doQueryReturn.listLength;
    var listHandle = doQueryReturn.listHandle;
    var numberOfChunks = Math.ceil(listLength / CHUNK_SIZE);

    log.info("iterateQueryList -listLength [" + listLength + "], chunkSize [" + CHUNK_SIZE + "], numberOfChunks [" + numberOfChunks + "]");

    try
    {
        for (var i = 0; i < numberOfChunks; i++)
        {
            var startIndex = i * CHUNK_SIZE;

            var endIndex = parseInt(startIndex) + parseInt(CHUNK_SIZE);
            endIndex = (listLength < endIndex ? listLength : endIndex) - 1;

            log.info("chunk [" + i + "] startIndex [" + startIndex + "], endIndex [" + endIndex + "]");

            // Get one chunk of the list of users
            var udsObjectList = caServiceDeskWS.getListValues(listHandle, startIndex, endIndex, attributes);

            log.info("getListValues returned " + udsObjectList.length + " entries");

            // Iterate over the groups in this chunk, processing them one at a time
            udsObjectList.forEach(callback);
        }
    }
    finally
    {
        log.info("Releasing doQuery resources for handle: " + listHandle);
        caServiceDeskWS.freeListHandles([listHandle]);
    }

    log.info("Exit - iterateQueryList");
}
